#!/usr/bin/env bash

set -euo pipefail

# CPU架构
arch=$(uname -m)
if [[ $arch != "x86_64" ]]; then
    echo "The architecture is unsupported! arch: $arch"
    exit 1
fi

# 脚本所在位置
sh_dir=$(cd "$(dirname "${BASH_SOURCE[0]}")" &> /dev/null && pwd)
cd $sh_dir

# 获取操作系统
source /etc/os-release

# 处理安装命令和版本
install_cmd=''
version_sep=''

if [[ $ID == "centos" || $ID_LIKE =~ "rhel" || $ID_LIKE =~ "fedora" ]]; then
    install_cmd='yum install -y'
    version_sep='-'
    mysql_version='5.7.44-1.el7'
    docker_version='20.10.24-3.el7'
    kubernetes_version='1.23.17-0.x86_64'
    java_package_name='java-1.8.0-openjdk-devel'
    java_version='1.8.0.412.b08-1.el7_9'
fi

if [[ -z $install_cmd ]]; then
    echo "Unsupported operating system! ID: $ID, ID_LIKE: $ID_LIKE"
    exit 1
fi

# 第一个参数是主机名
host=${1:-$(hostname)}

if [[ -z $host || $host =~ "localhost" ]]; then
  echo "hostname must not be empty or with a 'localhost' prefix!"
  exit 1
fi


# 第二个参数是角色，默认为非master
role=${2:-non-master}

echo "This is a $role node, will check and install Docker/Kubernetes"
sleep 10


# 0 先安装yum工具，方便后面配置
sudo yum install -y yum-utils

result=''

docker_ce_repo='https://mirrors.aliyun.com/docker-ce/linux/centos/docker-ce.repo'
docker_daemon_json='{
    "registry-mirrors":
    [
        "https://dockerproxy.com",
        "https://docker.m.daocloud.io",
        "https://docker.mirrors.ustc.edu.cn",
        "https://docker.nju.edu.cn",
        "https://registry.aliyuncs.com"
    ],
    "exec-opts":
    [
        "native.cgroupdriver=systemd"
    ]
}'

kubernetes_repo='[kubernetes]
name=Kubernetes
baseurl=https://mirrors.aliyun.com/kubernetes/yum/repos/kubernetes-el7-x86_64/
enabled=1
gpgcheck=1
gpgkey=https://mirrors.aliyun.com/kubernetes/yum/doc/yum-key.gpg https://mirrors.aliyun.com/kubernetes/yum/doc/rpm-package-key.gpg'

# 1 安装docker
if [[ -z $(command -v docker) ]]; then
    # 1.1 添加阿里云docker-ce的yum镜像仓库
    sudo yum-config-manager --add-repo $docker_ce_repo

    # 1.2 执行安装命令
    sudo $install_cmd docker-ce$version_sep$docker_version docker-ce-cli$version_sep$docker_version

    # 1.3 重新加载systemd配置
    sudo systemctl daemon-reload

    # 1.4 设置开机启动
    sudo systemctl enable --now docker

    # 1.5 配置docker
    echo "$docker_daemon_json" | sudo tee /etc/docker/daemon.json

    # 1.6 重启docker
    sudo systemctl restart docker

    result="$result \n docker [ installed ]"
else
    result="$result \n docker [ exists ]"
fi

sleep 5


# 2 安装kubernetes
if [[ -z $(command -v kubeadm) || -z $(command -v kubectl) || -z $(command -v kubelet) ]]; then
    # 2.1 先给节点设置名称
    sudo hostname $host

    # 2.2 添加阿里云kubernetes的yum镜像仓库
    echo "$kubernetes_repo" | sudo tee /etc/yum.repos.d/kubernetes.repo

    # 2.3 执行安装命令
    sudo $install_cmd kubeadm$version_sep$kubernetes_version kubectl$version_sep$kubernetes_version kubelet$version_sep$kubernetes_version

    # 2.4 重加载k8s配置，并设置开机启动
    sudo systemctl daemon-reload
    sudo systemctl enable --now kubelet

    result="$result \n kubernetes [ installed ]"
else
    result="$result \n kubernetes [ exists ]"
fi

sleep 5

# 3 master节点
if [[ $role == "master" ]]; then
    # 3.1 列出k8s所需镜像
    sudo kubeadm config images list

    # 3.2 使用kubeadm方式初始化master节点
    image_repository='registry.cn-hangzhou.aliyuncs.com/google_containers'
    service_cidr='10.96.0.0/12'
    pod_network_cidr='10.244.0.0/16'

    kubeadm_init_cmd="sudo kubeadm init \
                --ignore-preflight-errors=all \
                --kubernetes-version=v${kubernetes_version%-*} \
                --image-repository=${image_repository} \
                --service-cidr=${service_cidr} \
                --pod-network-cidr=${pod_network_cidr} \
                --v=5"

    # 3.3 执行初始化命令，并将内容输出到终端
    init_output=$($kubeadm_init_cmd | tee /dev/tty)
    # 3.4 从内容中找到join命令以及下一行，并写到join-kubernetes.sh中
    join_command=$(echo "$init_output" | grep 'kubeadm join' -A 1)
    echo "sudo $join_command" | tee $sh_dir/join-kubernetes.sh
    # 3.5 将k8s连接配置复制到用户主目录
    mkdir -p $HOME/.kube
    sudo cp -i /etc/kubernetes/admin.conf $HOME/.kube/config
    sudo chown $(id -u):$(id -g) $HOME/.kube/config
    # 从网络下载并flannel和dashboard编排文件
    wget https://s3-model.hinadt.com/HaishuuSoftware/kube-flannel.yml
    kubectl apply -f kube-flannel.yml
    wget https://s3-model.hinadt.com/HaishuuSoftware/k8s-dashboard-v2.5.1.yml
    kubectl apply -f k8s-dashboard-v2.5.1.yml
    # 创建dashboard admin账户，并赋予集群管理员角色
    kubectl create sa dashboard-admin -n kubernetes-dashboard
    kubectl create clusterrolebinding dashboard-cluster-admin \
            --clusterrole=cluster-admin \
            --serviceaccount=kubernetes-dashboard:dashboard-admin
    sleep 10
    dashboard_token=$(kubectl describe secret \
        $(kubectl get sa dashboard-admin -n kubernetes-dashboard \
            -o jsonpath="{.secrets[0].name}") -n kubernetes-dashboard \
        | awk '/token:/ {print $NF}')
    echo $dashboard_token | tee dashboard-admin-token
    # 预先创建命名空间
    kubectl create ns cloudeon
    # 去除master污点，允许pod部署
    kubectl taint no $(kubectl get no -o jsonpath="{.items[0].metadata.name}") \
        node-role.kubernetes.io/master:NoSchedule-

    # 4 安装jdk
    if [[ -z $(command -v java) ]]; then
        sudo $install_cmd $java_package_name$version_sep$java_version
        result="$result \n jdk [ installed ]"
    else
        result="$result \n jdk [ exists ]"
    fi
fi

# 最后打印结果
echo -e $result


