#!/usr/bin/env bash

# 安装软件包，先安装依赖，再安装本体
# Require:    local_install_cmd install_cmd
# Usage:      install_package <package_dir> <package_version>
# Argument1:  package_dir     软件包目录
# Argument2:  package_version 软件包版本
install_package() {
    local package_type=''
    if [[ $install_cmd =~ ^yum || $install_cmd =~ ^rpm || $install_cmd =~ ^dnf ]]; then
        package_type='rpm'
    elif [[ $install_cmd =~ ^dpkg || $install_cmd =~ ^apt ]]; then
        package_type='deb'
    fi

    if [[ -z $package_type ]]; then
        echo "Unsupported install command! local_install_cmd: $local_install_cmd"
        exit 1
    fi
    local package_path=$1/$2
    if [[ -e $package_path && -d $package_path ]]; then
        [ -d $package_path/dependencies ] && sudo $local_install_cmd $package_path/dependencies/*.$package_type
        sudo $local_install_cmd $package_path/*.$package_type
    else
        local pkg=${1##*/}
        echo "Local package path does not exists, will install '$pkg' from repo via internet!"
        sudo $install_cmd $pkg
    fi
}

ssh_cmd(){
    ssh -i "$NODE_SSH_KEY_PATH" "$@"
}

scp_cmd(){
    scp -i "$NODE_SSH_KEY_PATH" "$@"
}

# 通过awk分割行，并将参数赋给变量，如果为空则使用默认变量
# Require:    LINE
# Usage:      line_arg_assign_to_var <index_num> <variable_name> <default_var_name>
# Argument1:  index_num              下标数字
# Argument2:  variable_name          变量名
# Argument3:  default_var_name       默认变量名
line_arg_assign_to_var() {
    eval "$2=\$(echo \$LINE | awk '{print \$$1}')"
    if [[ $# -ge 3 ]]; then
        eval "$2=\${$2:-\$$3}"
    fi
}

# 将公钥拷贝到远程机器上
# Require:    node_ip node_port node_user node_pass
# Usage:      copy_public_key_to_remote <expect_script_path>
# Argument1:  expect_script_path        期望脚本路径
copy_public_key_to_remote() {
    if [[ -n $node_ip ]]; then
        # 执行期望脚本，将公钥复制到其他节点上，实现免密登录
        expect $1 "$NODE_SSH_KEY_PATH".pub $node_ip $node_port $node_user $node_pass
        local result=$?
        if [[ $result -eq 0 ]]; then
            echo "copy public key to remote machine succeeded! node_ip: $node_ip"
        else
            echo "copy public key to remote machine failed! node_ip: $node_ip"
        fi
    else
        echo "node ip is empty! node_ip: $node_ip"
    fi
}

# 将文件拷贝到远程机器上
# Require:    node_ip node_port node_user node_path
# Usage:      copy_file_to_remote <file_patterns>
# Argument1:  file_patterns       文件模式字符串，可传入多个参数
copy_file_to_remote() {
    local file=""
    for file in "$@"; do
        echo "$file"
        local scp_options=''

        if [[ ! -f "$file" ]]; then
            scp_options='-r'
        fi

        scp_cmd -P$node_port $scp_options $file $node_user@$node_ip:$node_path/
    done
}

run_script_on_remote() {
    ssh_cmd -n -p$node_port -n $node_user@$node_ip "bash $node_path/script/$1"
}

# 使用rsync将文件拷贝到远程机器上
# Require:    node_ip node_port node_user node_path
# Usage:      rsync_file_to_remote <file_patterns>
# Argument1:  file_patterns        文件模式字符串，可传入多个参数
rsync_file_to_remote() {
    local file=""
    for file in "$1"; do
        if [[ -z $2 ]]; then
            rsync -avz -e "ssh -i $NODE_SSH_KEY_PATH" $file $node_user@$node_ip:$node_path/
        else
            rsync -avz $2 $3 -e "ssh -i $NODE_SSH_KEY_PATH" $file $node_user@$node_ip:$node_path/
        fi
    done
}

# 遍历节点SSH信息
# Require:    NODE_SSH_INFO NODE_SSH_USER NODE_SSH_PASSWORD NODE_SSH_PORT NODE_SSH_PATH
# Usage:      retrieve_node_ssh_info <commands>
# Argument1:  commands               命令，可传入多个参数
retrieve_node_ssh_info() {
    local LINE=""
    local LINE_NUM=0
    while read -r LINE; do
        if [[ $LINE =~ ^# || -z $LINE ]]; then
            continue
        fi

        line_arg_assign_to_var 1 node_ip
        line_arg_assign_to_var 2 node_user NODE_SSH_USER
        line_arg_assign_to_var 3 node_pass NODE_SSH_PASSWORD
        line_arg_assign_to_var 4 node_port NODE_SSH_PORT
        line_arg_assign_to_var 5 node_path NODE_SSH_PATH

        eval "$@"
        LINE_NUM=$((LINE_NUM + 1))
    done < $NODE_SSH_INFO

    if [[ $LINE_NUM -lt 3 ]]; then
        echo "Cluster nodes size is less than 3, cannot do init all, please check $NODE_SSH_INFO!"
        exit 1
    fi
}

# 遍历节点SSH信息，并在执行命令后等待
# Require:    NODE_SSH_INFO NODE_SSH_USER NODE_SSH_PASSWORD NODE_SSH_PORT NODE_SSH_PATH
# Usage:      retrieve_node_ssh_info_in_background <commands>
# Argument1:  commands                             命令，可传入多个参数
retrieve_node_ssh_info_in_background() {
    local LINE=""
    local PID_LIST=""
    while read -r LINE; do
        if [[ $LINE =~ ^# || -z $LINE ]]; then
            continue
        fi

        line_arg_assign_to_var 1 node_ip
        line_arg_assign_to_var 2 node_user NODE_SSH_USER
        line_arg_assign_to_var 3 node_pass NODE_SSH_PASSWORD
        line_arg_assign_to_var 4 node_port NODE_SSH_PORT
        line_arg_assign_to_var 5 node_path NODE_SSH_PATH

        eval "$@ &"
        PID_LIST="$PID_LIST $node_ip-$!"
    done < $NODE_SSH_INFO

    for PID in $PID_LIST; do
        echo "wait $PID"
        wait ${PID##*-}
        echo "$PID is complete!"
    done
}

# 处理第一个节点的SSH信息
# Require:    NODE_SSH_INFO NODE_SSH_USER NODE_SSH_PASSWORD NODE_SSH_PORT NODE_SSH_PATH
# Usage:      handle_first_node_ssh_info <commands>
# Argument1:  commands                   命令，可传入多个参数
handle_first_node_ssh_info() {
    local LINE=""
    while read -r LINE; do
        if [[ $LINE =~ ^# || -z $LINE ]]; then
            continue
        fi

        line_arg_assign_to_var 1 node_ip
        line_arg_assign_to_var 2 node_user NODE_SSH_USER
        line_arg_assign_to_var 3 node_pass NODE_SSH_PASSWORD
        line_arg_assign_to_var 4 node_port NODE_SSH_PORT
        line_arg_assign_to_var 5 node_path NODE_SSH_PATH

        eval "$@"
        break
    done < $NODE_SSH_INFO
}

# 处理除第一个节点外的SSH信息
# Require:    NODE_SSH_INFO NODE_SSH_USER NODE_SSH_PASSWORD NODE_SSH_PORT NODE_SSH_PATH
# Usage:      handle_rest_node_ssh_info <commands>
# Argument1:  commands                  命令，可传入多个参数
handle_rest_node_ssh_info() {
    local LINE=""
    local NUM=0
    while read -r LINE; do
        if [[ $LINE =~ ^# || -z $LINE ]]; then
            continue
        fi
        NUM=$((NUM + 1))
        if [[ $NUM -eq 1 ]]; then
            echo "Skip first node!"
            continue
        fi

        line_arg_assign_to_var 1 node_ip
        line_arg_assign_to_var 2 node_user NODE_SSH_USER
        line_arg_assign_to_var 3 node_pass NODE_SSH_PASSWORD
        line_arg_assign_to_var 4 node_port NODE_SSH_PORT
        line_arg_assign_to_var 5 node_path NODE_SSH_PATH

        eval "$@"
    done < $NODE_SSH_INFO
}

# 倒计时，并在终端打印数字
# Usage:        count_down <num>
# Argument1:    num        倒计时秒数
count_down() {
    local num=$1
    while [ $num -gt 0 ]; do
        printf "\033[2K\r%d" "$num"
        sleep 1
        ((num--))
    done
    echo ""
}