#!/bin/bash

check_sudo_nopasswd() {
    # 如果当前用户是 root，则直接通过
    if [ "$(id -u)" -eq 0 ]; then
        echo "[OK] $(hostname), You are root."
        return 0
    fi
    
    # 检查是否有无密码 sudo 权限
    if sudo -n -l 2>/dev/null | grep -q "NOPASSWD"; then
        echo "[OK] $(hostname), You have passwordless sudo."
        return 0
    else
        echo "[ERROR] $(hostname), No passwordless sudo (NOPASSWD) found or no sudo privileges at all."
        exit 1
    fi
}

# 检测防火墙服务
check_firewall() {
    if systemctl is-active --quiet iptables; then
        echo "[ERROR] $(hostname),iptables firewall is running!"
        exit 1
    fi
    
    if systemctl is-active --quiet firewalld; then
        echo "[ERROR] $(hostname),firewalld firewall is running!"
        exit 1
    fi
    
    if systemctl is-active --quiet ufw; then
        echo "[ERROR] $(hostname),ufw firewall is running!"
        exit 1
    fi
    
    echo "[OK] $(hostname),No firewall services are running."
}

# 检测时钟同步服务
check_timesync() {
    if systemctl is-active --quiet ntpd; then
        echo "[OK] $(hostname),The ntpd time synchronization service is running."
        return 0
    fi
    
    if systemctl is-active --quiet chronyd; then
        echo "[OK] $(hostname),The chrony time synchronization service is running."
        return 0
    fi

    if systemctl is-active --quiet systemd-timesyncd; then
        echo "[OK] $(hostname),The systemd-timesyncd time synchronization service is running."
        return 0
    fi

    # 如果没有任何时钟同步服务在运行，返回 0
    echo "[ERROR] $(hostname),No time synchronization services detected!"
    exit 1
}

# 检测是否启用了swap
check_swap() {
    # 使用 swapon -s 命令检查是否有启用的 swap
    if sudo swapon -s | grep -q .; then
        echo "[ERROR] $(hostname),Swap is enabled. The script will terminate."
        exit 1
    fi
    echo "[OK] $(hostname),Swap is not enabled."
}

# 检测虚拟内存区域的值
check_virtual_memory() {
    # 从 /proc/meminfo 中获取总内存（以 kB 为单位）
    total_memory=$(grep MemTotal /proc/meminfo | awk '{print $2}')
    
    # 如果总内存小于 2000000 kB，则报错并退出
    if [ "$total_memory" -lt 2000000 ]; then
        echo "[ERROR] $(hostname),Total virtual memory is less than 2000000 kB (2 GB)."
        exit 1
    fi

    echo "[OK] $(hostname),Virtual memory is normal. Total memory: ${total_memory} kB."
}

# 检测透明大页是否启用
check_transparent_hugepage() {
    thp_status=$(cat /sys/kernel/mm/transparent_hugepage/enabled)
    if [[ "$thp_status" == *"[enabled]"* ]]; then
        echo "[ERROR] $(hostname),Transparent HugePages are enabled. The script will terminate."
        exit 1
    fi
    echo "[OK] $(hostname),Transparent HugePages are not enabled. The script will proceed."
}

# 检测系统最大打开文件句柄数
check_max_open_files() {
    max_open_files=$(ulimit -n)
    if [ "$max_open_files" -lt 65535 ]; then
        echo "[ERROR] $(hostname),The system's maximum number of open file handles is less than 65535. Current value: $max_open_files."
        exit 1
    fi

    echo "[OK] $(hostname),The system's maximum number of open file handles is normal: $max_open_files."
}

check_sudo_nopasswd
check_firewall
check_timesync
# check_swap
# check_virtual_memory
# check_transparent_hugepage
# check_max_open_files